    const moment = require('moment');
	const Stimulsoft = require("stimulsoft-reports-js");
	
	const StartTop = 0; //отправная точка колонок по top
	const StartLeft = 18; //отправная точка колонок по left ()
	const CellHeight = 0.6; //высота ячейки
	const DateCellWidth = 6; //ширина трех колонок
	const DetailCellWidth = 2; //ширина одной колонки
	const DaysOfWeek = ['воскресенье', 'понедельник', 'вторник', 'среда', 'четверг', 'пятница', 'суббота'];
	
	/**
     * Дополняет объект отчета динамическими данными: столбцы, заголовки, данные
     * @param {StiReport} report объект отчета
     * @param {string} datein_str дата начала периода
     * @param {string} dateout_str дата окончания периода
     */
    #dataBandReport(report, datein_str, dateout_str) {
        try {
            const page = report.pages.list[0];
            const dataSource = report.dataSources.list[0];

            const headerComponent = page.components.list.find(c => c.name === 'HeaderBandData');
            const dataComponent = page.components.list.find(c => c.name === 'DataBandData');

            const datein = moment(datein_str).startOf('days');
            const dateout = moment(dateout_str).startOf('days');
            let date = datein.clone();
            let numColumn = 1;

			//подсчет ширины страницы изначальная ширина + ширина колонки Итого + ширина колонки для дня * количество дней
            page.width = StartLeft + DetailCellWidth + dateout.diff(datein, 'days') * DateCellWidth;
            //Приведение высоты страницы к формату A4 (21х29,7)
            page.height = 21 * page.width / 29.7;

            let left = StartLeft;
            let top = StartTop;
            let stiText;

            while (date < dateout) {
                const numOfWeek = parseInt(date.format('d'));

                //добавляем колонки в Data Source
                this.#addColumnsToDataSource(numColumn, dataSource);

                //добавляем текстовые компоненты в Data Band для отображения данных
                this.#addDataToDataBand(left, numColumn, dataComponent);

                //добавляем текстовые компоненты в Header
                stiText = this.#initTextReport(left, top, DateCellWidth, CellHeight, date.format('DD.MM.YYYY'), Stimulsoft.Base.Drawing.StiTextHorAlignment.Center, Stimulsoft.Base.Drawing.StiVertAlignment.Center, false);
                headerComponent.components.add(stiText);
                top += CellHeight;
                stiText = this.#initTextReport(left, top, DateCellWidth, CellHeight, DaysOfWeek[numOfWeek], Stimulsoft.Base.Drawing.StiTextHorAlignment.Center, Stimulsoft.Base.Drawing.StiVertAlignment.Center, false);
                headerComponent.components.add(stiText);
                top += CellHeight;
                stiText = this.#initTextReport(left, top, DetailCellWidth, CellHeight, 'Приход', Stimulsoft.Base.Drawing.StiTextHorAlignment.Center, Stimulsoft.Base.Drawing.StiVertAlignment.Center, false);
                headerComponent.components.add(stiText);
                left += DetailCellWidth;
                stiText = this.#initTextReport(left, top, DetailCellWidth, CellHeight, 'Уход', Stimulsoft.Base.Drawing.StiTextHorAlignment.Center, Stimulsoft.Base.Drawing.StiVertAlignment.Center, false);
                headerComponent.components.add(stiText);
                left += DetailCellWidth;
                stiText = this.#initTextReport(left, top, DetailCellWidth, CellHeight, 'Присутствие', Stimulsoft.Base.Drawing.StiTextHorAlignment.Center, Stimulsoft.Base.Drawing.StiVertAlignment.Center, false);
                headerComponent.components.add(stiText);
                left += DetailCellWidth;
                top = 0;

                date = date.add(1, 'days');
                numColumn++;
            }

            //добавляем текстовый компонент итоговых данных в Data Band
            stiText = this.#initTextReport(left, 0, DetailCellWidth, CellHeight, '{data.sum}', Stimulsoft.Base.Drawing.StiTextHorAlignment.Right, Stimulsoft.Base.Drawing.StiVertAlignment.Center);
            dataComponent.components.add(stiText);

            //добавляем текстовый компонент заголовка Итого в Header
            stiText = this.#initTextReport(left, top, DetailCellWidth, CellHeight * 3, SummaryHeader, Stimulsoft.Base.Drawing.StiTextHorAlignment.Center, Stimulsoft.Base.Drawing.StiVertAlignment.Center);
            headerComponent.components.add(stiText);
        }
        catch (err) {
        }
    }
	
    /**
     * Добавляет колонку в DataSource отчета
     * @param {number} numColumn порядковый номер колонки
     * @param {StiDataSource} dataSource DataSource отчета
     */
    #addColumnsToDataSource(numColumn, dataSource) {
        let stiDataColumn = new Stimulsoft.Report.Dictionary.StiDataColumn(`col${numColumn}_date_start`, `col${numColumn}_date_start`);
        dataSource.columns.add(stiDataColumn);
        stiDataColumn = new Stimulsoft.Report.Dictionary.StiDataColumn(`col${numColumn}_date_end`, `col${numColumn}_date_end`);
        dataSource.columns.add(stiDataColumn);
        stiDataColumn = new Stimulsoft.Report.Dictionary.StiDataColumn(`col${numColumn}_attend_time_str`, `col${numColumn}_attend_time_str`);
        dataSource.columns.add(stiDataColumn);
    }

    /**
     * Добавляет данные в DataBand отчета
     * @param {number} left левая координата объекта текст
     * @param {number} numColumn порядковый номер колонки
     * @param {StiDataBand} dataComponent DataBand отчета
     */
    #addDataToDataBand(left, numColumn, dataComponent) {
        let stiText = this.#initTextReport(left, 0, DetailCellWidth, CellHeight, `{data.col${numColumn}_date_start}`, Stimulsoft.Base.Drawing.StiTextHorAlignment.Right, Stimulsoft.Base.Drawing.StiVertAlignment.Center);
        dataComponent.components.add(stiText);
        stiText = this.#initTextReport(left + DetailCellWidth, 0, DetailCellWidth, CellHeight, `{data.col${numColumn}_date_end}`, Stimulsoft.Base.Drawing.StiTextHorAlignment.Right, Stimulsoft.Base.Drawing.StiVertAlignment.Center);
        dataComponent.components.add(stiText);
        stiText = this.#initTextReport(left + DetailCellWidth * 2, 0, DetailCellWidth, CellHeight, `{data.col${numColumn}_attend_time_str}`, Stimulsoft.Base.Drawing.StiTextHorAlignment.Right, Stimulsoft.Base.Drawing.StiVertAlignment.Center);
        dataComponent.components.add(stiText);
    }

    /**
     * Создает объект Text для отчета
     * @param {number} left левая позиция
     * @param {number} top верхняя позиция
     * @param {number} width ширина
     * @param {number} height высота
     * @param {string} text текстовое содержание
     * @param {enum StiTextHorAlignment} horAlignment выравнивание по горизонтали
     * @param {enum StiVertAlignment} vertAlignment выравнивание по вертикали
     * @returns
     */
    #initTextReport(left, top, width, height, text, horAlignment, vertAlignment, growToHeight = true) {
        const stiText = new Stimulsoft.Report.Components.StiText();
        stiText.left = left;
        stiText.top = top;
        stiText.width = width;
        stiText.height = height;
        stiText.text = text;
        stiText.border = new Stimulsoft.Base.Drawing.StiBorder(Stimulsoft.Base.Drawing.StiBorderSides.All);
        stiText.horAlignment = horAlignment;
        stiText.vertAlignment = vertAlignment;
        stiText.canGrow = true;
        stiText.growToHeight = growToHeight;

        return stiText;
    }

